# Übung 1
# HFP - Einführung in die Methoden der empirischen Sozialforschung
# Author: Cindy Cheng, Mirco Schönfeld

##################################

##################################

# 1.1: Install R 
 # http://cran.r-project.org/bin/
 # works on all platforms
 # Unless you are compiling your own software, you will always want 'binary' distributions of software.

# 1.2: Install RStudio
	# https://www.rstudio.com
	# Note you must install R before you install RStudio to get RStudio to work

#1.3: So what is RStudio?
	# `RStudio is a free and open-source integrated development environment (IDE) for R'
		# Huh? Basically RStudio is like a big picture frame for R, where R is the picture. 
		# It organizes your R into 4 'frames':
			# 1) the R script (where you write out what you want R to do),
			# 2) the R console (where you tell R what you want it to do and where R answers to your commands), 
			# 3) a frame that toggles between 
					# a) the R environment: which catalogs the data or the variables that you have loaded or created in R and 
					# b) 'History' which catalogs all the R script that you have previously run
			# 4) a frame that toggles between 
					# a) `Files': a directory of the files and folders on your computer 
					# b) `Plots': Shows you any visualizations that you have asked R to create 
					# c) `Packages': Documents any user-contributed packages that you have loaded onto R 
					# d) `Help': Gives you the help page for any package or function that you have asked help for e) 'Viewer': Allows you to view any local web content


# 1.4 General Guideline for how to use RStudio

# 1) Open RStudio
# 2) Open a new  R Script (.R) file 
# 3) Write the commands that you would like R to do in your .R file
# 4) Run the command 
		# a) In order to run one line of code at a time 
    # 1) move your cursor to the line you want to run and 
    # 2) hit CTRL+RETURN (if you have Windows) or hit CMD+RETURN (if you have a Mac)
    # Try it on the following line

6 + 3

		# b) In order to run multiple lines of code 
		# 1) Highlight multiple lines of code with your mouse 
		# 2) hit CTRL+RETURN (if you have Windows) or hit CMD+RETURN (f you have a Mac)
		# Try it on the following lines

6 + 3
sqrt(6 + 3)


##################################

##################################
 
# Lets get started!

# 1.6 Doing simple arithmetic in R

5^2
6 + 4
10 - 23
8*4
29/3

# Sometimes it is necessary to keep the result of an equation to work with it later on. 
# To do so, we can store a result in a variable using <- as a notation. 
number <- 6 + 2
# The part left of the arrow is the name of the variable and the part on the right is what you store inside that variable. Once you stored something in a variable, you can access that something by using the variable's name. 
number
# You can choose whatever name you want. For example, "wrchslbrmpf" would be allowed, yet not recommended. It is always helpful to have names that actually tell you what is "inside". 
# Notice that something changed in the "Environment"-field above. The current "environment" of your commands now knows the word "number". Whenever you use this new word, R "replaces" that word with the value this variable stores.
# We can also update variables:
number <- number + 1
number
number2 <- number + 1
# and we can do something like this
sqrt(number)
# Here, R knows the variable "number" and therefore knows which data to use. And now?
sqrt(nmber)
# A typo...an error...! Oh no! Don't worry, get used to this. This happens very often. The most important thing to remember here is: READ THE ERROR MESSAGE! It helps you a lot. Now, apparently, R does not know the object "nmber". It should be a variable... But, R is correct - "nmber" is not listed in the Environment-field. What happens if the typo appears somewhere else?
sqt(number)
# Again - a helpful message. What if it is cryptic and impossible to understand? Copy, paste, and google it!
sqrt(number2)

# 1.7 R and sequences

# What is nice about R, and this is a point in which R differs from a lot of other programming concepts, is its concept of "vectorization". You will see very few cases in which you will work with single values inside variables like before. In fact, most of the time you will work with sequences of data:
c(1:10)
c(10:1)
c(2:25)
c(3,5,7,17)
c(3,3,3,17)
seq(from=2, to=10, by=2)
rep(42, 23)
rep(c(1:3), 5)
one_to_ten <- c(1:10)
one_to_ten
# The cool thing here are R's functions. Most of them work for sequences just as well!
sqrt(seq(from=2, to=10, by=2))
sqrt(c(3:9))
sum(c(4,5,10))
sum(one_to_ten)
min(one_to_ten)
max(one_to_ten)
mean(one_to_ten)

# And you can calculate with sequences, too! 
one_to_ten
one_to_ten + 1
one_to_ten * 2
one_to_ten ^ 3
one_to_ten / 4

greater_than_ten <- c(21:30)
one_to_ten
greater_than_ten
one_to_ten + greater_than_ten
one_to_ten * greater_than_ten

# R even helps you to calculate simple statistics about the sequence in a very eays manner:
summary(greater_than_ten)
# summary interprets the sequence as a distribution of values and describes the distribution via its min/max, its 1st and 3rd quantiles, and its mean and median. 
summary(c(0:100))
summary(c(5,28,3,10,32,17,9,1,24.5))
sum(5,28,3,10,32,17,9,1,24.5)

# We can access specific elements of sequences, as well. Therefore, we use square brackets
one_to_ten[4]
greater_than_ten[9]
# If we want more than one element:
one_to_ten[4:8]
greater_than_ten[c(1,3,7)]
greater_than_ten[c(9,3,6,1,10)]

# With that in mind, we can manipulate sequences
one_to_ten[4] <- 16
one_to_ten
greater_than_ten[10] <- 45
greater_than_ten
summary(greater_than_ten)

# One important thing regarding sequences: These kind of sequences always store the same kind of data! You can't mix, for example, words with numbers in one sequence!
c(3, "hello", 4.5)
# Here, R surrounds every entry with quotation marks - for R, every entry is a word.
words <- c(3, "hello", 4.5)
words * 3
# This error "non-numeric argument to binary operator" means: The asterisk expects two numbers to operate on. But you feed it with one "thing" that is no number, e.g. non-numeric. Your command can't be evaluated.


# R and matrices

# As a tool for linear algebra, R supports matrices very well. For us, they help us to understand how to "navigate" through data.
m <- matrix(data=c(1:12), nrow = 4, ncol = 3)
m
# This creates a matrix with 4 rows and 3 columns. The fields are filled with numbers from 1 to 12. Notice how the columns are filled first. 
m <- matrix(data=c(1:12), nrow = 4, ncol = 3, byrow=TRUE)
m
# This is how the rows are filled first. Keep this in mind for the first exercise ;-)
# Anyway, to inspect this matrix we can start by looking at the numbers of rows and columns. 
nrow(m)
ncol(m)

# Both can be seen by using dim(). Note here, the order of the two numbers. In R, specifying rows before columns is consistent throughout many operations.
dim(m)

# Navigating through a matrix often means to "subset". Therefore, we use an index that we write inside square brackets
m[,3]
# Wondering about the comma? Remember 
dim(m)
# R always uses (row, column). So, in this case, 
m[,3]
# means selecting ALL rows and the 3th column. If we are more interested in rows instead, we could write 
m[2,]

# By selecting single rows or single columns, R gives us sequences just like those seen before.
length(m[2,])
sum(m[2,])
length(m[,3])
mean(m[,3])
summary(m[1,])
m[3,]
m[3,] + 1
m[,2] * 2
m[4,] + m[2,]
m[,3] * m[,1]

# We can even use these sequences to change the matrix! If we need to change values stored in the second column of the matrix, we hand R a sequence with 4 values
m
m[,2] <- c(1:4)
m
# To update a row, use a sequence with 3 entries instead
m[1,] <- c(9:7)
m
# This will be helpful if you need to update some collected data with corrected values. 
m[3,1] <- 7.5
m

# What if we get the lengths of rows and columns mixed up?
m[1,] <- c(1:4)
# Just a helpful error message - no need to worry. The same goes for an arithmetic operation on sequences of different lengths
m[,3] * m[1,]


# One last aspect on those square brackets: We can put sequences inside there, too! First, lets create a bigger matrix
bigm <- matrix(data=c(1:50), nrow = 5, ncol = 10, byrow=TRUE)
bigm
# Now, let's get the 3rd to 7th column!
bigm[,c(3:7)]
# Or the 4th and 5th row
bigm[c(4,5),]


# 1.8 Work with data

# Here, we will work with a standard dataset that ships with R. Later in this course, you will learn how to read in your own data. For now, it is our goal to familiarize you with handling, navigating and inspecting data. 
# Therefore, we will focus on the "esoph"-dataset:
esoph

# Since this dataset ships with R, there is a documentation available. The following line works on every function or dataset that comes with R:
?esoph

# Ok, now we know what the dataset contains. Let's inspect the data. Most likely, the first couple of questions that you will want to ask are : 
	# 1) Does the data make sense? --> is the data valid?
		
		# Except now, you will rarely have the privilege of working with a completely clean dataset. 
		# Checking the data to make sure that, you have sensible values for your data is an important first step
		# For example, if somebody responds that they are 200 years old, you know that you have a problem in your data
		# Since cleaning collected data is a big topic, we will focus on this later in this course. 
	
	# 2) You will also want to know, what does the data say? ---> what are the descriptive statistics?
		
		# That is, for this cancer dataset, you might be curious about:
			# How many cases were reported at all?
			# How many cases of cancer were reported per age group?
		# ---> You will want to know how the data can be described

# inspect all the data
esoph
# ... or just peak at the top of the table - often helpful for laaaaaaaaarge datasets (several thousand rows)... 
head(esoph)
# and accordingly
tail(esoph, n = 4)

# what you can see looks like a table or a matrix and is called a dataframe. Just like matrices, it consists of a number of rows and a number of columns:
nrow(esoph)
# For dataframes, this number often corresponds to the number of observations, or subjects, or experiments. As you may remember from the documentation, this is the number of unique combinations of age group, alcohol, and tobacco consumption. 
# The number of columns on the other hand corresponds to the number of variables that describe one observation, subject, experiment, or combination. To show it use
ncol(esoph)

# Both can be seen by using dim() as well. Note here, the order of the two numbers is constistent.
dim(esoph)

sum(esoph[,4])
sum(esoph[c(10:24),4])
esoph[c(10:24),]

# have a look at the column's names, only:
names(esoph)
# these names can be used to work with single columns, only:
esoph$ncases
esoph[,4]
# this gives you all entries of the column ncases in the ordering in which they appear in the dataframe. In R, one such column of a dataframe is called a vector. You will often work with vectors. 
length(esoph$ncases) 
# Of course, this corresponds to the number of rows of the dataframe, whereas the following corresponds to the number of cancer patients and control group cases, instead:
sum(esoph$ncases)
sum(esoph$ncontrols)

# We saw the summary-command earlier. It works just as well on column-vectors of dataframes:
summary(esoph$ncases)
summary(esoph$ncontrols)
# It even works on the whole dataframe!
summary(esoph)
# Here, a summary is automatically calculated for every column of the dataframe. 
# Notice the difference in how the summaries of the columns look. ncases and ncontrols contain numerical data. That's why the summary gives basic insights into the distribution of values. The first three columns contain categorical data - so called "levels". R detects that there is a limited number of values that repeats very often. That's why for that column there are no basic statistics but counts of occurences of each level. We will cover levels in depth later in this course
